% This Matlab program controls maximum likelihood estimation for cointegrated case
% when both b1 and b2 are unknown
%
% Specify true parameter values
truea = [1;-1];
trueb = [0;.10];
trueomega = eye(2);
truechol = inv(chol(trueomega));
%
% Specify control parameters
ccontmain = 5;  % ccontmain = 4 means set b1 = trueb2 or b2 = trueb2 based on
                % likelihood ratio test
                % ccontmain = 2 means impose b2 = trueb2 regardless
                % ccontmain = 3 means impose a1 = truea1 regardless
                % ccontmain = 5 means set b1 = true, b2 = true b1, a1 = true a1,
                % or a2 = true a2 based on which has best likelihood 
ccont = 3;      % ccont = 1 means b1 = trueb2, others unrestricted
                % ccont = 2 means b2 = trueb2, others unrestricted
                % ccont = 3 means a1 = truea1, others unrestricted
                % ccont = 4 means a2 = truea2, others unrestricted
nitz = 10000;     % nitz is the number of simulations
a = zeros(2,nitz);   % column itz of a contains values of a vector for simulation # itz
b = zeros(2,nitz);   % column itz of b contains values of b vector for simulation # itz
vcount = zeros(1,nitz);  % vcount keeps track of which normalization for each sample
% following lines make sure same random numbers are generated each time
% program is run
dseed = 69323;
randn('state',dseed);

% ====================================================================================
% main control loop begins here
itz = 1;
while itz <= nitz;
%
% Generate data
% y will be a (2 x capt) matrix

capt = 50;
eps = randn(2,capt);
y = zeros(2,1);
ylag = y;
it = 1;
while it <= capt;
    ylag = ylag + trueb*(truea'*ylag) + eps(:,it);
    y = [y ylag];
it = it+1;
end;
dely = y(:,2:capt+1) - y(:,1:capt);  % This is the change in y 
ylag = y(:,1:capt);

% Calculate zeta0
svv = ylag*ylag'/capt;
suu = dely*dely'/capt;
suv = dely*ylag'/capt;
d = inv(svv)*suv'*inv(suu)*suv;
[vectors, values] = eig(d);
if values(1,1) < values(2,2);
    a1 = vectors(:,2);
else;
    a1 = vectors(:,1);
end;

a1 = a1 / sqrt(a1'*svv*a1);
zeta0 = suv*a1*a1';

if ccontmain == 4;
    % Find MLE for case of b1 = 0;
    y1 = dely(2,:)';
    x1 = ylag';
    pi1 = x1 \ y1;
    vee = dely(2,:)' - ylag'*pi1;
    vee = [dely(1,:)' vee];
    vee1 = sum(log(diag(vee'*vee/capt)));
    
    % Find MLE for case of b2 = 0;
    y1 = dely(1,:)';
    x1 = ylag';
    pi1 = x1 \ y1;
    vee = dely(1,:)' - ylag'*pi1;
    vee = [vee dely(2,:)'];
    vee2 = sum(log(diag(vee'*vee/capt)));
    
    % assign ccont based on better normalization
    if vee1 > vee2;
        ccont = 1;
        vcount(1,itz) = 1;
    else;
        ccont = 2;
    end;
elseif ccontmain == 5;
    % Find MLE for case of b1 = 0;
    y1 = dely(2,:)';
    x1 = ylag';
    pi1 = x1 \ y1;
    vee = dely(2,:)' - ylag'*pi1;
    vee = [dely(1,:)' vee];
    vee1 = sum(log(diag(vee'*vee/capt)));
    
    % Find MLE for case of b2 = 0;
    y1 = dely(1,:)';
    x1 = ylag';
    pi1 = x1 \ y1;
    vee = dely(1,:)' - ylag'*pi1;
    vee = [vee dely(2,:)'];
    vee2 = sum(log(diag(vee'*vee/capt)));
    
    % Find MLE for case of a1 = 0;
    y1 = dely(1,:)';
    x1 = ylag(2,:)';
    pi1 = x1 \ y1;
    vee3a = dely(1,:)' - ylag(2,:)'*pi1;
    vee3a = vee3a'*vee3a/capt;
    y2 = dely(2,:)';
    x2 = ylag(2,:)';
    pi2 = x2 \ y2;
    vee3b = dely(2,:)' - ylag(2,:)'*pi2;
    vee3b = vee3b'*vee3b/capt;
    vee3 = log(vee3a) + log(vee3b);
    
    % Find MLE for case of a2 = 0;
    y1 = dely(1,:)';
    x1 = ylag(1,:)';
    pi1 = x1 \ y1;
    vee4a = dely(1,:)' - ylag(1,:)'*pi1;
    vee4a = vee4a'*vee4a/capt;
    y2 = dely(2,:)';
    x2 = ylag(1,:)';
    pi2 = x2 \ y2;
    vee4b = dely(2,:)' - ylag(1,:)'*pi2;
    vee4b = vee4b'*vee4b/capt;
    vee4 = log(vee4a) + log(vee4b);
    
    
    veex = [1 vee1; 2 vee2; 3 vee3; 4 vee4];
    veex = sortrows(veex,2);
   
    ccont = veex(4,1);
    vcount(1,itz) = ccont;     
    
elseif ccontmain == 2;
    ccont = 2;
elseif ccontmain == 3;
    ccont = 3;
end;
    
    
% infer b and a from zeta0
if ccont == 1;
    b(1,itz) = trueb(2);
    a(1,itz) = zeta0(1,1)/b(1,itz);
    a(2,itz) = zeta0(1,2)/b(1,itz);
    b(2,itz) = zeta0(2,1)/a(1,itz);
elseif ccont == 2;
    b(2,itz) = trueb(2);
    a(1,itz) = zeta0(2,1)/b(2,itz);
    a(2,itz) = zeta0(2,2)/b(2,itz);
    b(1,itz) = zeta0(1,1)/a(1,itz);
elseif ccont == 3;
    a(1,itz) = truea(1);
    b(1,itz) = zeta0(1,1)/a(1,itz);
    b(2,itz) = zeta0(2,1)/a(1,itz);
    a(2,itz) = zeta0(2,2)/b(2,itz);
elseif ccont == 4;
    a(2,itz) = truea(2);
    b(1,itz) = zeta0(1,2)/a(2,itz);
    b(2,itz) = zeta0(2,2)/a(2,itz);
    a(1,itz) = zeta0(1,1)/b(1,itz);
end;
    
itz = itz+1;
end;
% main control loop ends here
% =========================================================================

% summarize results
disp('a vector mean, standard deviations');
mean(a')
std(a')
disp('b vector mean, standard deviations');
mean(b')
std(b')

if ccontmain > 3;
subplot('Position',[0.2 0.80 0.6 0.15]);
[h f xx] = plotdens(a(1,:)');
plot(xx,f)
axis([-2,2,0,3])
title('Sampling density of a1')

subplot('Position',[0.2 0.55 0.6 0.15]);
[h f xx] = plotdens(a(2,:)');
plot(xx,f)
axis([-3,3,0,2])
title('Sampling density of a2')

subplot('Position',[0.2 0.3 0.6 0.15]);
[h f xx] = plotdens(b(2,:)');
plot(xx,f)
axis([-2,2,0,15])
title('Sampling density of b2')

subplot('Position',[0.2 0.05 0.6 0.15]);
[h f xx] = plotdens(b(1,:)');
plot(xx,f)
axis([-2,2,0,10])
title('Sampling density of b1')

elseif ccontmain == 3;
    
subplot('Position',[0.2 0.7 0.6 0.2]);
[h f xx] = plotdens(a(2,:)');
plot(xx,f)
title('Sampling density of a2')

subplot('Position',[0.2 0.4 0.6 0.2]);
[h f xx] = plotdens(b(2,:)');
plot(xx,f)
title('Sampling density of b2')

subplot('Position',[0.2 0.1 0.6 0.2]);
[h f xx] = plotdens(b(1,:)');
plot(xx,f)
title('Sampling density of b1')

elseif ccontmain == 2;

subplot('Position',[0.2 0.7 0.6 0.2]);
[h f xx] = plotdens(a(1,:)');
plot(xx,f)
title('Sampling density of a1')

subplot('Position',[0.2 0.4 0.6 0.2]);
[h f xx] = plotdens(a(2,:)');
plot(xx,f)
title('Sampling density of a2')

subplot('Position',[0.2 0.1 0.6 0.2]);
[h f xx] = plotdens(b(1,:)');
plot(xx,f)
title('Sampling density of b1')
    
end;
hcount = zeros(1,nitz);
icount = 1;
while icount < nitz;
    if vcount(1,icount) == 1;
        hcount(1,icount) = 1;
    end;
    icount = icount+1;
end;
disp('number of incorrectly normalized samples')
sum(hcount)

