% (A) For each sample, (1) generate artificial data from from an i.i.d. mixture model with both means
%     and variances unknown (2 states); (2) use MCMC method to simulate posterior density from these data.
% (B) Simulate many samples to get the average sample-mean, standard deviation, 25%, 50%, 68%,
%     and 90% coverage probabilities.
%
%
% Copyright (c) Hamilton, Waggoner, and Zha, May 2002.  See HWZ Normalization Notes, pp. 1-4g.




%------------------- Values manually initialized ---------------------
T = 50;  % Sample size.
indxNmlab = [0 1 0 0]  % Normalization on labeling states.
                     % If all zeros, no normalization.
                     % If indxNmlab(1), normalization of mu1>mu2>mu3>....
                     % If indxNmlab(2), normalization of p1>p2>p3>....
                     % If indxNmlab(3), distance normalization (closest to the peak).
                     % If indxNmlab(4), asymmetric prior (e.g., truncated normal).
gmuSigma = [
  5            0
  0       5
  ];   % Prior covariance matrix of [mu1,mu2], imported from the results generated by mix2a_prior.m.

% =====================================================
%  delete these lines if don't want looping 
%Att = [10 20 50 100];
Att = [20 100];
Btt = eye(4);
%for Itt = 1:1;
    %if Itt == 1;
    %    gmuSigma = [
     %  100            0
      %   0       100
      %    ];   % Prior covariance matrix of [mu1,mu2], imported from the results generated by mix2a_prior.m.
      % else;
        gmuSigma = [
       5            0
         0       5
         ];   % Prior covariance matrix of [mu1,mu2], imported from the results generated by mix2a_prior.m.
         %end;
 %    indxNmlab = Btt(Itt,:);
  %   for T = Att;
  %        'normalization'
  %        indxNmlab
  %        'sample size'
  %        T
  %        'gmuSigma'
  %        gmuSigma
          
yser = zeros(T,1);   % Overwritten.
seednumber = 69323;     % if 0, random state at each clock time
nbuffer = 250;  % 100;        % A block or buffer of draws (buffer) that is saved to the disk (not memory)
nSamples = 4*nbuffer %4*nbuffer   % Number of samples to get classical criteria (such as coverage probabilities).
ndraws1=2*nbuffer %40*nbuffer        % 1st part of Monte Carlo draws to be thrown away.
ndraws2=20*nbuffer %20*nbuffer %2000*nbuffer         % 2nd part of Monte Carlo draws to be kept.
%**** Prior parameters.
gmubar = 0*ones(2,1);  % Prior mean of [mu1,mu2].
std2_inv = 1.0/sqrt(gmuSigma(2,2));  %For computing the jumping ratio in the Metropolis-Hastings algorithm.
scf_mh = 1.5;  %For a truncated normal prior.: Scale factor for a proposal density in the Metropolis-Hastings algorithm with scf_mf^2*Sigma where Sigma is VarianceMatrix.
galpha = [1;1];  % Prior parameters for the Dirichlet of p and 1-p.
%**** True parameters for generating artificial data with sigma1=sigma2=1.
m = 0;
gdelta = 1; %.5; %0.5;
gmu = [m+gdelta;m-gdelta];   % Ture means [mu1; mu2].
gsigma = [1;1];  %[1;5];   % True standard deviations for nStates.
nStates = length(gmu);   % Number of states.
p = 0.8  %0.8
gtheta = [gmu;p];   % A vector of the values of true parameters.
%**** Settings for finding the ML (posterior peak).
crita0 = sqrt(eps);   % Sims's csminwel's convergence ((only one) criterion.
nit = 100;    % Maximum number of iterations for csminwell.m to meet crita0.
x0 = [gmu;log(p/(1-p))];    % Initial and overwritten.  ML estimate.
%x0 = [2;5;0.8]
H0 = eye(length(x0));
nPerms=factorial(nStates);   % Number of complete permutations.
gthetagbsM = zeros(nPerms,nStates+1);  % Overwritten. 3: [mu1, mu2, p].


%------------------- Values automaticallyser initialized ---------------------
%
%***** Get all the permutations of [1:nStates].
tmpd = fullfact([nStates:-1:2]);
            % To permute completely on [1:n], use the product of transpositions as
            %     (n,i_n)...(j,i_j)...(1,1_1) where 1<=i_n<=n and (j,i_j) means the transposition
            %     of the jth and i_jth positions (if j=i_j, stay in the same position) and
            %     where (1,1_1) is whatever is left because i_1 cannot be greater than 1 so that
            %     1=1_1.
Spermall = zeros(nPerms,nStates);  % Overwritten.  All permutations of n states.
for ki=1:nPerms
   tmpS = [1:nStates];
   scount=0;
   for si=nStates:-1:2
      scount=scount+1;
      tmp = tmpS;
      if (si>tmpd(ki,scount))
         tmpS(tmpd(ki,scount)) = tmp(si);
         tmpS(si) = tmp(tmpd(ki,scount));
      end
   end
   Spermall(ki,:) = tmpS;
end

if seednumber   % To have the same draws of gamrnd and randn for each while loop.
   randn('state',seednumber);
   rand('state',seednumber);
else
   randn('state',fix(100*sum(clock)));
   rand('state',fix(100*sum(clock)));
end



%-------------------------- Repetitions for, say, coverage probabilities begin here --------------------
%Ps2y2 = zeros(nStates,1);   % Overwritten.
%logPy2s2 = zeros(nStates,1);  % Overwritten.
Tk = zeros(nStates,1);  % Overwritten. T_k.
yksum = zeros(nStates,1); % Overwritten y_k^s.
xdraws = zeros(length(gtheta),ndraws2);   % Overwritten.
AveML = zeros(size(xdraws,1),1);   % Cumulative.  Average ML estimate.
AveMean = zeros(size(xdraws,1),1);   % Cumulative.  Average mean estimate.
AveMedian = zeros(size(xdraws,1),1);   % Cumulative.  Average median estimate.
AveVar = zeros(size(xdraws,1),1);   % Cumulative.  Average variance of posterior distribution.
AveMSE = zeros(size(xdraws,1),1);   % Cumulative.  Average MSEs.
CovProb50 = 0;   % Cumulative.  Nominal coverage probabilities for 50%.
CovProb68 = 0;   % Cumulative.  Nominal coverage probabilities for 68%.
CovProb90 = 0;   % Cumulative.  Nominal coverage probabilities for 90%.
LengthProb50 = 0;   % Cumulative.  Nominal coverage probabilities for 50%.
LengthProb68 = 0;   % Cumulative.  Nominal coverage probabilities for 68%.
LengthProb90 = 0;   % Cumulative.  Nominal coverage probabilities for 90%.
%
timebeg=clock;
ncntbuf = 0;
ndegcases = 0;
jdraws = [];
for sami=1:nSamples
   %**** Generate artificial data.
   t1=sum(rand(T,1)<p);
   yser(1:t1) = gmu(1) + gsigma(1)*randn(t1,1);
   yser(t1+1:T) = gmu(2) + gsigma(2)*randn(T-t1,1);

   if indxNmlab(3)   % Normalization of HWZ (distance normalization).
      %------------------- Get the posterior estimate. ---------------------
      [fhat,x1,gradhat,Hhat,itct,fcount,exitflag] = ...
         csminwel('ftd_mix2a_post',x0,H0,'',crita0,nit,T,yser,gmubar,gmuSigma,galpha,gsigma);  % <<>>0 ftd_mix2mca_post
      if x1(1) < x1(2)
         gmuhat = [x1(2); x1(1)];
         phat = 1 - exp(x1(3))/(1+exp(x1(3)));
         gthetahat = [gmuhat;phat];
      else
         gmuhat = x1(1:2);
         phat = exp(x1(3))/(1+exp(x1(3)));
         gthetahat = [gmuhat;phat];
      end

%      options = optimset('Display','off','LargeScale','off','TolX',crita0,'TolFun',crita0,'TolCon',crita0);   % off:  BFGS method.  ('GradObj','on')
%      [x1,fhat,exitflag,output] = fminunc(@ftd_mix2a_post,x0,options,T,yser,gmubar,gmuSigma,galpha,gsigma);
%      gmuhat = x1(1:2);
%      phat = exp(x1(3))/(1+exp(x1(3)));
%      gthetahat = [gmuhat;phat];

      %--------------- Get Hessian at the peak ------------------
      %* stepsize
      %grdh = eps^(1/3) * (max([abs(xhat) ones(size(xhat,1),1)]'))' .* (abs(xhat) ./ xhat);
      grdh = 1e-04;
      %
      %  Hess = fn_hesscd('ftd_mix2mca_post',x1,grdh,T,yser,nStates,gvarsigma,galpha);
      %           % inv(Omega) where Omega is the covariance matrix at the posterior peak.
      %  check = chol(Hess);  % Check to make sure that it's positive definite.
      Hessg = zeros(length(x1));   % Cumulative.  Hessian computed from the gradient.  See (4-52) p. 115 in Greene.
      for ti=1:T
         grdnt = fn_gradcd('ftd_mix2a_post',x1,grdh,1,yser(ti),gmubar,gmuSigma,galpha,gsigma);  % <<>>0  n-by-1.
         Hessg = Hessg + grdnt*grdnt';
      end
      %
      JacobM = [
         1 0 0
         0 1 0
         0 0 1/(phat*(1-phat))
            ];  % For (mu1,mu2,p).  Jacobian matrix.
      Hessp = JacobM'*Hessg*JacobM;
   end


   %------------------- Simulating S_T and {gmu; p} from the posterior. ---------------------
   %------------------- See HWZ Normalization Notes pp. 1-4g. ---------------------
   %
   %**** Starting point.
   gmugbs = gmu;
   Pgbs = [p;1-p];
   tic

   %--------------- Results from the 1st loop to be discarded.
   Urand = rand(T,ndraws1);   % Used for drawing states S_T below.
   cnt_zeros1=0;  % Cummulative.
   normcdfold = normcdf((gmugbs(1)-gmubar(2))*std2_inv,0,1);
   for drawi=1:ndraws1
      %========== Draw S_T conditional on Y_T and theta={gmu, p}.  See (1b) and (2) on pp.1-2.
      Ps2y2 = Pgbs(1) ./ ( Pgbs(1) + Pgbs(2)*exp( -0.5*(gmugbs(2)^2-gmugbs(1)^2) + yser*(gmugbs(2)-gmugbs(1)) ) );  %T-by-1 for State 1..
      S_T = (Urand(:,drawi)<Ps2y2);
      Tk(1) = sum(S_T);
      Tk(2) = T-Tk(1);
      yksum(1) = sum(yser .* S_T);
      yksum(2) = sum(yser .* (1-S_T));
      if (min(Tk)==0)
         cnt_zeros1 = cnt_zeros1 + 1;
      end


      %========== Posterior draws of the transition matrix P (Dirichlet).  See (7) p.3
      Prndgam=gamrnd(galpha+Tk,ones(nStates,1));
      Pgbs = Prndgam ./ repmat(sum(Prndgam,1),[nStates 1]);
      Pgbs(find(~isfinite(Pgbs))) = 1;   % In case of a degenerate case.

      %========== Posterior draws of mu1 and mu2.  See p.4c (c.f. (8) on p.3).
      if indxNmlab(4)    %Metropolis-Hastings draw at this Gibbs step.  See pp.3a-3c.
         Dinv_Tk = diag(Tk);  % Inverse of D.
         gmuSigmatld_inv = Dinv_Tk+inv(gmuSigma);  % Inverse of posterior covariance of mu's. Note tld: tilde.
         gmuSigmatld_inv_diag = diag(gmuSigmatld_inv);  % Inverse of posterior covariance of mu's. Note tld: tilde.
         gmuStd_diag = scf_mh ./ sqrt(gmuSigmatld_inv_diag);  %Standard deviations in the proposal density.
         gmutld = (1.0 ./ gmuSigmatld_inv_diag) .* (yksum + gmubar ./ diag(gmuSigma));

         gmuJump = gmugbs + gmuStd_diag .* randn(2,1);
         if (gmuJump(2) < gmuJump(1))
            normcdfJump = normcdf((gmuJump(1)-gmubar(2))*std2_inv ,0,1);
            if ((normcdfold <= 2.225e-307) | (normcdfJump <= 2.225e-307))
                gmugbs = gmuJump;   %Always jumps in this generate case.
                normcdfold = normcdfJump;
                disp(' ')
                disp('Warning: the value of normcdf is an underflow so the approximation of always jumping may be bad!')
                disp('Number of of degenerate cases so far is:')
                ndegcases = ndegcases + 1
            else
               qratio = (normcdfold/normcdfJump)*exp(-0.5*( sum(((gmuJump-gmutld).^2-(gmugbs-gmutld).^2).*gmuSigmatld_inv_diag) ));
               if (rand < min([1.0 qratio]))
                  gmugbs = gmuJump;   %Jumps or accepts the proposal draw.
                  normcdfold = normcdfJump;
               end
            end
         end
      else
         Dinv_Tk = diag(Tk);  % Inverse of D.
         gmuSigmatld_inv = Dinv_Tk+inv(gmuSigma);  % Inverse of posterior covariance of mu's. Note tld: tilde.
         gmutld = gmuSigmatld_inv \ (yksum+gmuSigma\gmubar);  % Posterior mean of mu's.  Note tld: tilde.
         gmugbs = gmutld + chol(gmuSigmatld_inv)\randn(2,1);  % A Gibbs draw of mu's.
      end


%========== Different normalization rules.
      %if isempty(find(indxNmlab))   % No normalization with random permutation of states.
      if indxNmlab(1)   % Normalization of mu1>mu2.
         [jnk_sort,nmlzorder]=sort(gmugbs);
         gmugbs1=gmugbs;
         Pgbs1 = Pgbs;
         gmugbs = gmugbs1(nmlzorder');
         Pgbs = Pgbs1(nmlzorder');
         xdraws(:,drawi) = [flipud(gmugbs);Pgbs(2)];   % Save the draws of [mu1, mu2, p].
      elseif indxNmlab(2)   % Normalization of p1>p2.
         [jnk_sort,nmlzorder]=sort(Pgbs);
         gmugbs1=gmugbs;
         Pgbs1 = Pgbs;
         gmugbs = gmugbs1(nmlzorder');
         Pgbs = Pgbs1(nmlzorder');
         xdraws(:,drawi) = [flipud(gmugbs);Pgbs(2)];   % Save the draws of [mu1, mu2, p].
      elseif indxNmlab(3)   % Wald Normalization.
         for si=1:nPerms
            gmugbs1 = gmugbs(Spermall(si,:));
            Pgbs1 = Pgbs(Spermall(si,:));
            gthetagbsM(si,:) = [gmugbs1' Pgbs1(1)];
         end
         DiffM = (gthetagbsM-repmat(gthetahat',[nPerms 1]));
         Gtheta1 = DiffM*Hessp;  % N-by-n, where N is factorial(nStates) and n is the number of variables.
         distmes = sum((Gtheta1 .* DiffM),2);  % N-by-1.  Distance measures.
         [jnk,distInx] = min(distmes);
         xdraws(:,drawi) = gthetagbsM(distInx,:)';   % Save the draws of [mu1, mu2, p].
      end
% ===================================================================
   end
   clear Urand;

   %--------------- Results from the 2nd loop to be saved.
   Urand = rand(T,ndraws2);   % Used for drawing states S_T below.
   cnt_zeros2=0; %Cumulative.
   JumpRatio=0;  %Cumulative.
   normcdfold = normcdf((gmugbs(1)-gmubar(2))*std2_inv,0,1);
   for drawi=1:ndraws2
      %========== Draw S_T conditional on Y_T and theta={gmu, p}.  See (1b) and (2) on pp.1-2.
      Ps2y2 = Pgbs(1) ./ ( Pgbs(1) + Pgbs(2)*exp( -0.5*(gmugbs(2)^2-gmugbs(1)^2) + yser*(gmugbs(2)-gmugbs(1)) ) );  %T-by-1 for State 1..
      S_T = (Urand(:,drawi)<Ps2y2);
      Tk(1) = sum(S_T);
      Tk(2) = T-Tk(1);
      yksum(1) = sum(yser .* S_T);
      yksum(2) = sum(yser .* (1-S_T));
      if (min(Tk)==0)
         cnt_zeros2 = cnt_zeros2 + 1;
      end

      %========== Posterior draws of the transition matrix P (Dirichlet).  See (7) p.3
      Prndgam=gamrnd(galpha+Tk,ones(nStates,1));
      Pgbs = Prndgam ./ repmat(sum(Prndgam,1),[nStates 1]);
      Pgbs(find(~isfinite(Pgbs))) = 1;   % In case of a degenerate case.

      %========== Posterior draws of mu1 and mu2.  See p.4c (c.f. (8) on p.3).
      if indxNmlab(4)    %Metropolis-Hastings draw at this Gibbs step.  See pp.3a-3c.
         Dinv_Tk = diag(Tk);  % Inverse of D.
         gmuSigmatld_inv = Dinv_Tk+inv(gmuSigma);  % Inverse of posterior covariance of mu's. Note tld: tilde.
         gmuSigmatld_inv_diag = diag(gmuSigmatld_inv);  % Inverse of posterior covariance of mu's. Note tld: tilde.
         gmuStd_diag = scf_mh ./ sqrt(gmuSigmatld_inv_diag);  %Standard deviations in the proposal density.
         gmutld = (1.0 ./ gmuSigmatld_inv_diag) .* (yksum + gmubar ./ diag(gmuSigma));

         gmuJump = gmugbs + gmuStd_diag .* randn(2,1);
         if (gmuJump(2) < gmuJump(1))
            normcdfJump = normcdf((gmuJump(1)-gmubar(2))*std2_inv ,0,1);
            if ((normcdfold <= 2.225e-307) | (normcdfJump <= 2.225e-307))
                gmugbs = gmuJump;   %Always jumps in this generate case.
                normcdfold = normcdfJump;
                JumpRatio = JumpRatio + 1;
                disp(' ')
                disp('Warning: the value of normcdf is an underflow so the approximation of always jumping may be bad!')
                disp('Number of of degenerate cases so far is:')
                ndegcases = ndegcases + 1
            else
               qratio = (normcdfold/normcdfJump)*exp(-0.5*( sum(((gmuJump-gmutld).^2-(gmugbs-gmutld).^2).*gmuSigmatld_inv_diag) ));
               if (rand < min([1.0 qratio]))
                  gmugbs = gmuJump;   %Jumps or accepts the proposal draw.
                  normcdfold = normcdfJump;
                  JumpRatio = JumpRatio + 1;
               end
            end
         end
         xdraws(:, drawi) = [gmugbs; Pgbs(1)];
      else
         Dinv_Tk = diag(Tk);  % Inverse of D.
         gmuSigmatld_inv = Dinv_Tk+inv(gmuSigma);  % Inverse of posterior covariance of mu's. Note tld: tilde.
         gmutld = gmuSigmatld_inv \ (yksum+gmuSigma\gmubar);  % Posterior mean of mu's.  Note tld: tilde.
         gmugbs = gmutld + chol(gmuSigmatld_inv)\randn(2,1);  % A Gibbs draw of mu's.
      end


      %========== Different normalization rules.
      %if isempty(find(indxNmlab))   % No normalization with random permutation of states.
      if indxNmlab(1)   % Normalization of mu1>mu2.
         [jnk_sort,nmlzorder]=sort(gmugbs);
         gmugbs1=gmugbs;
         Pgbs1 = Pgbs;
         gmugbs = gmugbs1(nmlzorder');
         Pgbs = Pgbs1(nmlzorder');
         xdraws(:,drawi) = [flipud(gmugbs);Pgbs(2)];   % Save the draws of [mu1, mu2, p].
      elseif indxNmlab(2)   % Normalization of p1>p2.
         [jnk_sort,nmlzorder]=sort(Pgbs);
         gmugbs1=gmugbs;
         Pgbs1 = Pgbs;
         gmugbs = gmugbs1(nmlzorder');
         Pgbs = Pgbs1(nmlzorder');
         xdraws(:,drawi) = [flipud(gmugbs);Pgbs(2)];   % Save the draws of [mu1, mu2, p].
      elseif indxNmlab(3)   % Wald Normalization.
         for si=1:nPerms
            gmugbs1 = gmugbs(Spermall(si,:));
            Pgbs1 = Pgbs(Spermall(si,:));
            gthetagbsM(si,:) = [gmugbs1' Pgbs1(1)];
         end
         DiffM = (gthetagbsM-repmat(gthetahat',[nPerms 1]));
         Gtheta1 = DiffM*Hessp;  % N-by-n, where N is factorial(nStates) and n is the number of variables.
         distmes = sum((Gtheta1 .* DiffM),2);  % N-by-1.  Distance measures.
         [jnk,distInx] = min(distmes);
         xdraws(:,drawi) = gthetagbsM(distInx,:)';   % Save the draws of [mu1, mu2, p].
      end
   end
   clear Urand;
   endminutes=toc/60;

   %========= Compute the final objects such as mean, coverage probalities, etc.  =================
   ncntbuf = ncntbuf + 1;
   if (indxNmlab(3) & ncntbuf==100)  % Normalization of HWZ (distance normalization).
      AveML = AveML + gthetahat;
      disp('(a) True parameters. (b) ML estimates.  (c) Convergence flag (=0 or 1 for Sims program and 1 for Matlab)')
      [gtheta gthetahat]
      exitflag
   end
   PntMean = mean(xdraws,2);
   AveMean = AveMean + PntMean;
   AveMedian = AveMedian + median(xdraws')';
   AveVar = AveVar + mean((xdraws-repmat(PntMean,[1 ndraws2])).^2,2);
   AveMSE = AveMSE + (PntMean-gtheta).^2;  % <<>>0 mean(xdraws,2) or gthetahat, etc.
   intvl50 = prctile(xdraws',[25 75])';  % <<>>0  number of parameters-by-2.
   CovProb50 = CovProb50 + (gtheta>intvl50(:,1)) .* (gtheta<intvl50(:,2));
   LengthProb50 = LengthProb50 + intvl50(:,2) - intvl50(:,1);
   intvl68 = prctile(xdraws',[16 84])';  % <<>>0  number of parameters-by-2.
   CovProb68 = CovProb68 + (gtheta>intvl68(:,1)) .* (gtheta<intvl68(:,2));
   LengthProb68 = LengthProb68 + intvl68(:,2) - intvl68(:,1);
   intvl90 = prctile(xdraws',[5 95])';  % <<>>0  number of parameters-by-2.
   CovProb90 = CovProb90 + (gtheta>intvl90(:,1)) .* (gtheta<intvl90(:,2));
   LengthProb90 = LengthProb90 + intvl90(:,2) - intvl90(:,1);


   if (ncntbuf==100)
      ncntbuf = 0;  %Reset to 0.
      disp(' ')
      disp('(a) Total # of samples. (b) # of samples so far. (c) Minutes for each sample. (d) Total # of MC draws (draws2) for each sample.'),
      [nSamples sami endminutes ndraws2]
      if indxNmlab(4)
         disp('Jumping ratio (percent) for MH algorithm')
         JumpRatio = 100*JumpRatio/ndraws2
      end
   end
%   if sami > ndraws1
       jdraws = [jdraws xdraws];
%   end
   %  disp(' ')
   %  str1='disp(';
   %  str2 = ['log marginal likelihood value is  ' num2str(logMarLHun0)];
   %  eval([str1 'str2' ')'])

   %endminutes
end
AveML=AveML/nSamples
AveMean=AveMean/nSamples
Bias=AveMean - gtheta
AveMedian=AveMedian/nSamples
AveVar=AveVar/nSamples
AveMSE=AveMSE/nSamples
RootofAveMSE=sqrt(AveMSE)
CovProb50=(CovProb50/nSamples)*100
LengthProb50=LengthProb50/nSamples
CovProb68=(CovProb68/nSamples)*100
LengthProb68=LengthProb68/nSamples
CovProb90=(CovProb90/nSamples)*100
LengthProb90=LengthProb90/nSamples
disp('Numbers of one of the states not drawn for both burn_in and usable blocks for the last sample')
[cnt_zeros1 cnt_zeros2]
if indxNmlab(4)
   disp('Total number of underflow cases when computing normcdf for all samples and all MC posterior draws (incl. burn-in blocks):')
   ndegcases
end
%
endhours=etime(clock,timebeg)/60/60  %'Interrupted at 1.0e-1';

%save outa_covprob indxNmlab T seednumber nSamples ndraws1 ndraws2 gmubar gmuSigma galpha ...
 %             m gdelta gmu gsigma nStates p gtheta ...
   %           AveML AveMean AveMedian AveVar AveMSE CovProb50 CovProb68 CovProb90
%end;
%end;
