function [ff,xx] = robust_dens(x,h,xlow,xhigh,nobs)
% Code to calculate a robust kernel density using triangular kernels
% written by James Hamilton 7/26/03
%  INPUTS
%     x = column vector of data
%     h = bandwidth
%     xlow = lowest value for which density is to be evaluated
%     xhigh = highest value
%     nobs = number of points between xlow and xhigh at which
%            density is to be evaluated
%  OUTPUTS
%    xx = set of points at which density was evaluated
%    ff = value of density at each point

% Determine points at which density is to be evaluated 
xx = linspace(xlow,xhigh,nobs)';
ff = zeros(nobs,1);

for i = 1:1:nobs;
    xk = x - xx(i);
    kk = max(0,1-abs(xk/h));
    ff(i) = (1/h)*mean(kk);
end;
