
#include "filesetup.h"
#include "impulse.h"
#include "var.h"
#include "m_array.h"
#include "matrix.h"
#include "histo.h"
#include "StringMatrix.h"
#include "parameterization.h"
#include "wald.h"
#include "pi.h"

void FindMaxMin(TMatrixHistogram *h, int i, int j, PRECISION *min, PRECISION *max, PRECISION tol, int *first)
{
  PRECISION new_min, new_max, prob;
  int k;

  prob=(PRECISION)(h->low[i][j])/(PRECISION)(h->sample_size);
  if (prob >= tol)
    new_min=h->Min[i][j];
  else
    {
      for (k=0; k < h->intervals; k++)
	{
	  prob+=(PRECISION)(h->freq[i][j][k])/(PRECISION)(h->sample_size);
	  if (prob >= tol) break;
	}
      new_min=h->Min[i][j] + k*(h->Max[i][j] - h->Min[i][j])/(PRECISION)(h->intervals);
    }

  prob=(PRECISION)(h->high[i][j])/(PRECISION)(h->sample_size);
  if (prob >= tol)
    new_max=h->Max[i][j];
  else
    {
      for (k=h->intervals-1; k >= 0; k--)
	{
	  prob+=(PRECISION)(h->freq[i][j][k])/(PRECISION)(h->sample_size);
	  if (prob >= tol) break;
	}
      new_max=h->Min[i][j] + (k+1)*(h->Max[i][j] - h->Min[i][j])/(PRECISION)(h->intervals);
    }

  if (*first)
    {
      *first=0;
      *min=new_min;
      *max=new_max;
    }
  else
    {
      if (new_min < *min) *min=new_min;
      if (new_max > *max) *max=new_max;
    }
}


static char *var[3]={"Quantity","Price","Weather"};
static char *shock[3]={"Demand","Supply","Weather"};
void TrueImpulseResponse(FILE *f_out, file_setup_type *setup, int horizon)
{
  int i, j, k;
  TMatrix *Impulse;
  Impulse=ImpulseResponseArray((TMatrix*)NULL,setup->A_true,setup->F_true,setup->n_lag,horizon+1);

  for (j=0; j < setup->n_var; j++)
    for (i=0; i < setup->n_var; i++)
      {
	fprintf(f_out,"\n%s to %s shock\n",var[i],shock[j]);
	for (k=0; k <= horizon; k++) fprintf(f_out,"%lf\n",Impulse[k][i][j]);
      }
  FreeMatrixArray(Impulse);
  PrintFileSetup(f_out,setup);
}

void PrintImpulseHistogram(FILE *f_out, file_setup_type *setup, TMatrixHistogram **h, int horizon, int bins)
{
  TStringVector headers;
  TMatrix X;
  int i, j, k, m;
 
  headers=ParameterizationHeaders(setup);
  fprintf(f_out,"%s\n",headers[0]);
   
  for (k=0; k <= horizon; k++)
    for (j=0; j < setup->n_var; j++)
      for (i=0; i < setup->n_var; i++)
	{
	  //fprintf(f_out,"/* Response of variable %d to shock in equation %d at horizon %d */\n",i+1,j+1,k); 
          fprintf(f_out,"%s to %s, horizon %d\n",var[i],shock[j],k);
	  X=PlotMatrixHistogramAuto(i,j,bins,h[k]);
          if (X)
	    for (m=0; m < M_ROW(X); m++)
	      fprintf(f_out,"%lf,%lf\n",X[m][0],X[m][1]);
	  //PrintMatrix(f_out,X,"%.3lf,");
	  FreeMatrix(X);  
	}

  PrintFileSetup(f_out,setup);

  FreeStringVector(headers);
}

void PrintParameterizationHistogram(FILE *f_out, file_setup_type *setup, TVectorHistogram *h, int bins)
{
  TStringVector headers;
  TMatrix X;
  int j, v_dim=ParameterizationDimension(setup);
 
  headers=ParameterizationHeaders(setup);
 
  for (j=0; j < v_dim; j++)
    {
      fprintf(f_out,"\n%s Parameterization\t%s\n",headers[0],headers[j+1]);
      X=PlotVectorHistogramAuto(j,bins,h);
      PrintMatrix(f_out,X,"%le\t");
      FreeMatrix(X);
    }

  PrintFileSetup(f_out,setup);

  FreeStringVector(headers);
}

#define NUMBER_PERCENTILES 3
void PrintPercentiles(FILE *f_out, file_setup_type *setup, TMatrixHistogram **h, int horizon)
{
  TStringVector headers;
  TMatrix X, *True;
  PRECISION p[NUMBER_PERCENTILES]={0.1,0.5,0.9};
  int i, j, k, m;
 
  headers=ParameterizationHeaders(setup);
  fprintf(f_out,"%s\n",headers[0]);

  if (setup->parameterization == 4)
    True=ImpulseResponseArrayPi((TMatrix*)NULL,setup->A_true,setup->F_true,setup->n_lag,horizon+1);
  else
    True=ImpulseResponseArray((TMatrix*)NULL,setup->A_true,setup->F_true,setup->n_lag,horizon+1);

  for (j=0; j < setup->n_var; j++)
    for (k=0; k < setup->n_var; k++)
      {
	fprintf(f_out,"/* Response of variable %d to shock in equation %d */\n",k+1,j+1);
	for (i=0; i <= horizon; i++) fprintf(f_out,"%c%lf",i ? ',' : ' ',True[i][k][j]);
	fprintf(f_out,"\n");
	for (m=0; m < NUMBER_PERCENTILES; m++)
	  {
	    for (i=0; i <= horizon; i++)
	      fprintf(f_out,"%c%lf",i ? ',' : ' ',Precentile(p[m],0,h[i]->freq[k][j],h[i]->Min[k][j],h[i]->Max[k][j],
							     h[i]->intervals,h[i]->sample_size));
	    fprintf(f_out,"\n");
	  }
      }

  FreeStringVector(headers);
}
#undef NUMBER_PERCENTILES

void PrintContoursList(FILE *f_out, file_setup_type *setup, TMatrixHistogram **h, int horizon, int bins)
{
  TStringVector headers;
  TMatrix X;
  PRECISION min_out, max_out;
  int i, j, k, m, first;
  char *vars[3]={"Quantity","Price","Weather"};
  char *shocks[3]={"Demand","Supply","Weather"};
 
  headers=ParameterizationHeaders(setup);

  fprintf(f_out,"%s\n",headers[0]);
  for (j=0; j < setup->n_var; j++)
    for (i=0; i < setup->n_var; i++)
      {
	fprintf(f_out,"\n%s to %s shock\n",vars[i],shocks[j]);
	first=1;
	for (k=0; k <= horizon; k++) FindMaxMin(h[k],i,j,&min_out,&max_out,0.005,&first);
	for (k=0; k <= horizon; k++)
	  {
	    X=PlotMatrixHistogram(i,j,min_out,max_out,bins,h[k]);
	    if (k == 0)
	      {
		for (m=0; m < M_ROW(X); m++) fprintf(f_out,",%lf",X[m][0]);
		fprintf(f_out,"\n");
	      }
	    fprintf(f_out,"%d",k);
	    for (m=0; m < M_ROW(X); m++) fprintf(f_out,",%lf",X[m][1]);
	    fprintf(f_out,"\n");
	    FreeMatrix(X); ;
	  }
      }

  PrintFileSetup(f_out,setup);

  FreeStringVector(headers);
}

/*
  type meaning
*/
#define OUTPUT_MLE             0x00000001
#define OUTPUT_SETUP           0x00000002
#define OUTPUT_PARAMS          0x00000100
#define OUTPUT_PARAMS_FULL     0x00000200
#define OUTPUT_IMPULSE_DATA    0x00000400
#define OUTPUT_PERCENTILES     0x00000800
#define OUTPUT_PARM_HISTO_VAR  0x00000020
#define OUTPUT_PARM_HISTO_FIX  0x00000040
#define OUTPUT_IMPULSE_HISTO   0x00000080

void Average(FILE *f_out, file_setup_type *setup, int type, int horizon, int bins)
{
  int i, j, k, r, s, t, v_dim;
  TMatrix Data=(TMatrix)NULL, Exg=(TMatrix)NULL, *Impulse,
    A, F, A_hat, F_hat, A_ref, F_ref,
    IM_true=(TMatrix)NULL, IM_hat=(TMatrix)NULL;
  TVector v, max, min;
  T_SVAR *svar;
  TMatrixHistogram **h_impulse;
  TVectorHistogram *h_parm_var, *h_parm_fix;
  TStringVector headers;

  v=CreateVector(v_dim=ParameterizationDimension(setup));

  h_impulse=(TMatrixHistogram**)malloc((horizon+1)*sizeof(TMatrixHistogram*));
  if (!h_impulse) return;
  for (i=0; i <= horizon; i++)
    h_impulse[i]=CreateMatrixHistogram(setup->n_var,setup->n_var,1000,HISTOGRAM_VARIABLE);

  h_parm_var=CreateVectorHistogram(v_dim,1000,HISTOGRAM_VARIABLE);

  h_parm_fix=CreateVectorHistogram(v_dim,1000,HISTOGRAM_FIXED);
  min=InitializeVector(CreateVector(v_dim),-10.0);
  max=InitializeVector(CreateVector(v_dim),10.0);
  SetMaxMinVectorHistogram(min,max,h_parm_fix);
  FreeVector(min);
  FreeVector(max);

  Impulse=CreateMatrixArray(horizon+1);
  for (i=0; i <= horizon; i++)
    Impulse[i]=CreateMatrix(setup->n_var,setup->n_var);

  if (setup->n_exg > 0)
    Exg=InitializeMatrix(CreateMatrix(setup->n_obs,1),1.0);  

  A=CreateMatrix(setup->n_var,setup->n_var);
  F=setup->n_pre ? CreateMatrix(setup->n_pre,setup->n_var) : (TMatrix)NULL;

  A_hat=CreateMatrix(setup->n_var,setup->n_var);
  F_hat=setup->n_pre ? CreateMatrix(setup->n_pre,setup->n_var) : (TMatrix)NULL;

  A_ref=CreateMatrix(setup->n_var,setup->n_var);
  F_ref=setup->n_pre ? CreateMatrix(setup->n_pre,setup->n_var) : (TMatrix)NULL;

  switch (setup->parameterization)
    {
    case 2: // Beta
      A_ref[0][0]=1.0; A_ref[0][1]=1.0; A_ref[0][2]=0.0;
      A_ref[1][0]=0.0; A_ref[1][1]=0.0; A_ref[1][2]=0.0;
      A_ref[2][0]=0.0; A_ref[2][1]=0.0; A_ref[2][2]=1.0;
      break;
    case 3: // Eta
      A_ref[0][0]=0.0; A_ref[0][1]=1.0; A_ref[0][2]=0.0;
      A_ref[1][0]=1.0; A_ref[1][1]=0.0; A_ref[1][2]=0.0;
      A_ref[2][0]=0.0; A_ref[2][1]=0.0; A_ref[2][2]=1.0;
      break;
    }

  setup->seed=initialize_generator(setup->seed);

  /* Print Headers */
  if (type & OUTPUT_PARAMS)
    {
      headers=ParameterizationHeaders(setup);
      fprintf(f_out,"%s\n",headers[0]);
      for (i=1; i <= v_dim; i++) fprintf(f_out,"%s%s",(i > 1) ? "\t" : "",headers[i]);
      fprintf(f_out,"\n");
      FreeStringVector(headers);
    }
  if (type & OUTPUT_PARAMS_FULL)
    {
      headers=ParameterizationHeaders(setup);
      fprintf(f_out,"%s\n",headers[0]);
      for (i=1; i <= v_dim; i++) fprintf(f_out,"%s%s",(i > 1) ? "\t" : "",headers[i]);
      for (k=1; k <= setup->n_lag; k++)
	for (j=0; j < setup->n_var; j++)
	  for (i=0; i < setup->n_var; i++)
	    fprintf(f_out,"\tA%d(%d,%d)",k,i+1,j+1);
      if (setup->n_exg)
	for (j=0; j < setup->n_var; j++)
	  fprintf(f_out,"\tC(%d)",j+1);
      fprintf(f_out,"\n");
      FreeStringVector(headers);
    }
  if (type & OUTPUT_IMPULSE_DATA)
    {
      headers=ParameterizationHeaders(setup);
      fprintf(f_out,"%s at horizon %d\n",headers[0],horizon);
      if (setup->parameterization > 0)
        for (j=0; j < setup->n_var; j++)
          for (i=0; i < setup->n_var; i++)
            fprintf(f_out,"%sResponse of %s to %s shock",(i+j > 0) ? "\t" : "",var[i],shock[j]);
      else
        for (j=0; j < setup->n_var; j++)
          for (i=0; i < setup->n_var; i++)
            fprintf(f_out,"%sResponse of variable %d to shock %d",(i+j > 0) ? "\t" : "",i+1,j+1);
      fprintf(f_out,"\n");
      FreeStringVector(headers);
    }

  /* 
      Outer loop 
       Generate data, setup svar structure and compute MLE.
       Compute reference matrics (A_ref and F_ref)
  */
  for (r=0; r < setup->outer_loop; r++)
    if (!(Data=GenerateData(setup)) || !(svar=SetupFromStructure(setup,Data))
	|| !Maximize(A_hat,100000,1E-6,1.0,RANDOM_STARTING_VALUE,svar))
      {
        if (Data)
	  {
	    FreeMatrix(Data);
	    if (svar) Destruct_SVAR(svar);
	  }
	printf("Error finding MLE\n");
	r--;
	continue;
      }
    else
      {
	printf("%d\n",r+1);
	switch (setup->parameterization)
	  {
	  case 1: //WZ
	    EquateMatrix(A_ref,A_hat);
	    Normalize_WZ(A_ref,setup->A_true);
	    break;
	  case 5: // Wald (1st derivative)
	    FreeMatrix(IM_true);
	    IM_true=InformationMatrix_1stDerivative(setup->A_true,setup->F_true,Data,Exg,setup->n_lag);
	    EquateMatrix(A_ref,A_hat);
	    GetF_hat(F_ref,svar,A_ref);
	    NormalizeWald(A_ref,F_ref,setup->A_true,setup->F_true,IM_true);
	    break;
	  case 6: // Wald (2nd derivative)
	    FreeMatrix(IM_true);
	    IM_true=InformationMatrix_2ndDerivative(setup->A_true,setup->F_true,Data,Exg,setup->n_lag);
	    EquateMatrix(A_ref,A_hat);
	    GetF_hat(F_ref,svar,A_ref);
	    NormalizeWald(A_ref,F_ref,setup->A_true,setup->F_true,IM_true);
	    break;
	  }
	FreeMatrix(Data);
	Destruct_SVAR(svar);

        /*
           Inner loop
             Generate data, setup svar structure and compute MLE.
             Normalize MLE (A_hat and F_hat) using reference matrix
        */
	for (s=0; s < setup->inner_loop; s++)
	  if (!(Data=GenerateData(setup)) || !(svar=SetupFromStructure(setup,Data))
	      || !Maximize(A_hat,100000,1E-6,1.0,RANDOM_STARTING_VALUE,svar))
	    {
	      if (Data)
		{
		  FreeMatrix(Data);
		  if (svar) Destruct_SVAR(svar);
		}
	      printf("Error finding MLE\n");
	      s--;
	      continue;
	    }
	  else
	    {
              /* Normalize MLE */
	      switch (setup->parameterization)
		{
		case 0: // No normalization
		  GetF_hat(F_hat,svar,A_hat);
		  break;
		case 1: //WZ
		  Normalize_WZ(A_hat,A_ref);
		  GetF_hat(F_hat,svar,A_hat);
		  break;
 	        case 2: // Beta
		  Normalize_Traditional(A_hat,A_ref);
		  GetF_hat(F_hat,svar,A_hat);
		  break;
	        case 3: // Eta
		  Normalize_Traditional(A_hat,A_ref);
		  GetF_hat(F_hat,svar,A_hat);
		  break;
		case 4:
		  GetF_hat(F_hat,svar,A_hat);
		  break;
		case 5: // Wald (1st derivative)
		  GetF_hat(F_hat,svar,A_hat);
		  NormalizeWald(A_hat,F_hat,A_ref,F_ref,IM_true);
		  FreeMatrix(IM_hat);
		  IM_hat=InformationMatrix_1stDerivative(A_hat,F_hat,Data,Exg,setup->n_lag);
		  break;
		case 6: // Wald (2nd derivative)
		  GetF_hat(F_hat,svar,A_hat);
		  NormalizeWald(A_hat,F_hat,A_ref,F_ref,IM_true);
		  FreeMatrix(IM_hat);
		  IM_hat=InformationMatrix_2ndDerivative(A_hat,F_hat,Data,Exg,setup->n_lag);
		  break;
		default:
		  printf("Unsupported parameterization type\n");
		  getchar();
		  exit(0);
		}

	      /* MLE parameters */
              if (type & OUTPUT_MLE)
		{
		  Parameterization(A_hat,v,setup->parameterization);
		  if (type & OUTPUT_PARAMS) PrintVector(f_out,v,"%lg ");
		  if (type & OUTPUT_PARAMS_FULL)
		    {
		      for (i=0; i < v_dim; i++) fprintf(f_out,"%lg ",v[i]);
		      for (k=0; k < setup->n_lag; k++)
			for (j=0; j < setup->n_var; j++)
			  for (i=0; i < setup->n_var; i++)
			    fprintf(f_out,"%lg ",F_hat[k*setup->n_var+i][j]);
		      if (setup->n_exg)
			for (j=0; j < setup->n_var; j++)
			  fprintf(f_out,"%lg ",F_hat[setup->n_lag*setup->n_var][j]);
		      fprintf(f_out,"\n");
		    }
		  if (type & OUTPUT_PARM_HISTO_VAR) AddVectorObservation(v,h_parm_var);
		  if (type & OUTPUT_PARM_HISTO_FIX) AddVectorObservation(v,h_parm_fix);
	       
                  /* MLE impulse response */
		  if (type & OUTPUT_IMPULSE_HISTO)
		    {
		      if (setup->parameterization == 4)
			ImpulseResponseArrayPi(Impulse,A_hat,F_hat,svar->n_lag,horizon+1);
		      else
			ImpulseResponseArray(Impulse,A_hat,F_hat,svar->n_lag,horizon+1);
		      for (i=0; i <= horizon; i++)
			AddMatrixObservation(Impulse[i],h_impulse[i]);
		      if (type & OUTPUT_IMPULSE_DATA)
			{
			  for (j=0; j < setup->n_var; j++)
			    for (i=0; i < setup->n_var; i++)
			      fprintf(f_out,"%lg ",Impulse[horizon][i][j]);
			  fprintf(f_out,"\n");
			}
		    }
		}

	      /* 
                 Parameter draws 
                   draw parameters and normalizes
              */
	      RandomInitialValue(A,100,svar);
	      for (t=0; t < setup->parameter_draws; t++)
		{
                  Simulate_A(A,svar);
		  /* Normalize */
                  switch (setup->parameterization)
		    {
		    case 0: // No normalization
		      Simulate_F(A,F,svar);
		      break;
		    case 1: //WZ
		      Normalize_WZ(A,A_hat);
		      Simulate_F(A,F,svar);
		      break;
		    case 2: // Beta
		    case 3: // Eta
		      Normalize_Traditional(A,A_ref);
		      Simulate_F(A,F,svar);
		      break;
		    case 4: // Pi
		      Simulate_F(A,F,svar);
		      break;
		    case 5: // Wald (1st derivative)
		    case 6: // Wald (2nd derivative)
		      Simulate_F(A,F,svar);
		      NormalizeWald(A,F,A_hat,F_hat,IM_hat);
		      break;
		    default:
		      printf("Unsupported parameterization type\n");
		      getchar();
		      exit(0);
		    }
		  Parameterization(A,v,setup->parameterization);
		  if (type & OUTPUT_PARAMS) PrintVector(f_out,v,"%lg ");
		  if (type & OUTPUT_PARAMS_FULL)
		    {
		      for (i=0; i < v_dim; i++) fprintf(f_out,"%lg ",v[i]);
		      for (k=0; k < setup->n_lag; k++)
			for (j=0; j < setup->n_var; j++)
			  for (i=0; i < setup->n_var; i++)
			    fprintf(f_out,"%lg ",F[k*setup->n_var+i][j]);
		      if (setup->n_exg)
			for (j=0; j < setup->n_var; j++)
			  fprintf(f_out,"%lg ",F[setup->n_lag*setup->n_var][j]);
		      fprintf(f_out,"\n");
		    }
		  if (type & OUTPUT_PARM_HISTO_VAR) AddVectorObservation(v,h_parm_var);
		  if (type & OUTPUT_PARM_HISTO_FIX) AddVectorObservation(v,h_parm_fix);
		  if (type & OUTPUT_IMPULSE_HISTO)
		    {
		      if (setup->parameterization == 4)
			ImpulseResponseArrayPi(Impulse,A,F,svar->n_lag,horizon+1);
		      else
			ImpulseResponseArray(Impulse,A,F,svar->n_lag,horizon+1);
		      for (i=0; i <= horizon; i++)
			AddMatrixObservation(Impulse[i],h_impulse[i]);
		      if (type & OUTPUT_IMPULSE_DATA)
			{
			  for (j=0; j < setup->n_var; j++)
			    for (i=0; i < setup->n_var; i++)
			      fprintf(f_out,"%lg ",Impulse[horizon][i][j]);
			  fprintf(f_out,"\n");
			}
		    }
		}

	      FreeMatrix(Data);
	      Destruct_SVAR(svar);
	    }
      }

  FreeMatrix(A);
  FreeMatrix(F);
  FreeMatrix(A_hat);
  FreeMatrix(F_hat);
  FreeMatrix(A_ref);
  FreeMatrix(F_ref);
  FreeMatrixArray(Impulse);
  FreeVector(v);
  FreeMatrix(IM_true);
  FreeMatrix(IM_hat);
  FreeMatrix(Exg);

  //PrintParameterizationHistogram(f_out,setup,h_parm_var,bins);

  //PrintParameterizationHistogram(f_out,setup,h_parm_fix,bins);

  if (type & OUTPUT_IMPULSE_HISTO)
    {
      if (type & OUTPUT_PERCENTILES)
	PrintPercentiles(f_out,setup,h_impulse,horizon);
      else
	PrintImpulseHistogram(f_out,setup,h_impulse,horizon,bins);
    }
    

  //PrintContoursList(f_out,setup,h_impulse,horizon,bins);

  if (type & OUTPUT_SETUP) PrintFileSetup(f_out,setup);


  for (i=0; i < horizon; i++) FreeMatrixHistogram(h_impulse[i]);
  free(h_impulse);

  FreeVectorHistogram(h_parm_var);
  FreeVectorHistogram(h_parm_fix);
}

int main(int n_arg, char **args)
{
  FILE *f_out, *f_in;
  char *filename;
  int  horizon, type; 
  file_setup_type *setup;

  SetMatrixErrorTerminate(0);
  SetMatrixErrorVerbose(0);

  if (ParseCommandLine(n_arg,args,'h') > 0)
    {
      printf("\nCommand line options\n"
	     "  -i : parameter input filename.  Default value parms.dat\n"
	     "  -o : output filename.  Default value output.dat\n"
	     "  -s : seed value.  Default value from input file\n"
	     "  -O : number outer passes.  Default value from input file\n"
	     "  -I : number inner passes.  Default value from input file\n"
	     "  -P : number parameter draws.  Default value from input file\n"
	     "  -H : horizon.  Default value 10\n"
	     "  -p : parameterization type.  Default value from input file\n"
	     "        0) No normalization\n"
	     "        1) WZ\n"
	     "        2) Beta\n"
	     "        3) Eta\n"
	     "        4) Pi\n"
	     "        5) Wald (1st derivative)\n"
	     "        6) Wald (2nd derivative)\n"
	     "  -t : Output type.  Default value 1\n"
	     "        1) Contemporaneous parameters\n"
	     "        2) All parameters\n"
	     "        3) Impulse responses at given horizon\n"
             "        4) Histogram of impulse responses at given horizon\n"
	     "  -S : Print setup values at end of file.  Default value 1\n"
	     "        0) Do not print\n"
	     "        1) print values\n"
	     "  -h : this help screen.\n");
      exit(0);
    }

  filename=ParseCommandLine_String(n_arg,args,'o',"output.dat");
  if (!(f_out=fopen(filename,"wt")))
    {
      fclose(f_in);
      printf("Unable to create %s\n",filename);
      exit(0);
    }

  filename=ParseCommandLine_String(n_arg,args,'i',"params.dat");
  if (!(f_in=fopen(filename,"rt")))
    {
      printf("Unable to open %s\n",filename);
      exit(0);
    }
  setup=SetupFromFile(f_in);
  fclose(f_in);

  setup->parameterization=ParseCommandLine_Integer(n_arg,args,'p',setup->parameterization);
  setup->seed=ParseCommandLine_Integer(n_arg,args,'s',setup->seed);
  setup->outer_loop=ParseCommandLine_Integer(n_arg,args,'O',setup->outer_loop);
  setup->inner_loop=ParseCommandLine_Integer(n_arg,args,'I',setup->inner_loop);
  setup->parameter_draws=ParseCommandLine_Integer(n_arg,args,'P',setup->parameter_draws);
  horizon=ParseCommandLine_Integer(n_arg,args,'H',10);

  type=ParseCommandLine_Integer(n_arg,args,'S',1) ? OUTPUT_SETUP : 0;
  switch (ParseCommandLine_Integer(n_arg,args,'t',1))
    {
    case 1: type|=OUTPUT_PARAMS; break;
    case 2: type|=OUTPUT_PARAMS_FULL; break;
    case 3: type|=OUTPUT_IMPULSE_HISTO | OUTPUT_IMPULSE_DATA; break;
    case 4: type|=OUTPUT_IMPULSE_HISTO; break;
      //case 4: type|=OUTPUT_IMPULSE_HISTO | OUTPUT_PERCENTILES; break;
    }

  Average(f_out,setup,type,horizon,100);
  //TrueImpulseResponse(f_out,setup,horizon);

  fclose(f_out);

  return 0;
}
