

#include "matrix.h"
#include "m_array.h"
#include "StringMatrix.h"
#include "histo.h"
#include "var.h"
#include "filesetup.h"
#include "wald.h"
#include "parameterization.h"

void PrintHistogramVector(FILE *f, char **header, TMatrix data)
{
 TMatrix Plot, TransposePlot;
 int i;
 int bins=40;
 
 fprintf(f,"%s\n",header[0]);

 for (i=0; i < M_ROW(data); i++)
  {
   //Plot=data_histogram_uniform_width((TMatrix)NULL,data[i],M_COL(data),bins);
   Plot=data_histogram((TMatrix)NULL,data[i],M_COL(data),bins);
   TransposePlot=Transpose((TMatrix)NULL,Plot);
   fprintf(f,"%s\n",header[i+1]);
   PrintMatrix(f,TransposePlot,"%le\t");
   FreeMatrix(Plot);
   FreeMatrix(TransposePlot);
  }

 free(header);
}

TStringVector BetaParameterizationHeader(void)
{
 return CreateStringVectorFromDelimited("Beta\tbeta\tgamma\th\td1\td2\td3",'\t',0);
}
void BetaParameterization(TMatrix A, TVector v)
{
 v[0]=-A[1][0]/A[0][0];        // beta
 v[1]=-A[1][1]/A[0][1];        // gamma
 v[2]=-A[2][1]/A[0][1];        // h
 v[3]=1.0/(A[0][0]*A[0][0]);   // d1
 v[4]=1.0/(A[0][1]*A[0][1]);   // d2
 v[5]=1.0/(A[2][2]*A[2][2]);   // d3
}

TStringVector EtaParameterizationHeader(void)
{
 return CreateStringVectorFromDelimited("Eta\teta\tgamma\th\td1'\td2\td3",'\t',0);
}
void EtaParameterization(TMatrix A, TVector v)
{
 v[0]=-A[0][0]/A[1][0];        // eta
 v[1]=-A[1][1]/A[0][1];        // gamma
 v[2]=-A[2][1]/A[0][1];        // h
 v[3]=1.0/(A[1][0]*A[1][0]);   // d1'
 v[4]=1.0/(A[0][1]*A[0][1]);   // d2
 v[5]=1.0/(A[2][2]*A[2][2]);   // d3
}

TStringVector PiParameterizationHeader(void)
{
 return CreateStringVectorFromDelimited("Pi\tpi1\tpi2\tsigma11\tsigma22\tsigma23\tsigma33",'\t',0);
}
void PiParameterization(TMatrix A, TVector v)
{
 PRECISION d=1.0/(A[0][0]*A[1][1] - A[0][1]*A[1][0]);
 v[0]=-A[2][1]*A[1][0]*d;                     // pi1
 v[1]=A[2][1]*A[0][0]*d;                      // pi2
 d*=d;
 v[2]=(A[1][1]*A[1][1]+A[1][0]*A[1][0])*d;    // sigma11
 v[3]=(A[0][1]*A[0][1]+A[0][0]*A[0][0])*d;    // sigma22
 v[4]=-(A[1][1]*A[0][1]+A[1][0]*A[0][0])*d;   // sigma12
 v[5]=1.0/(A[2][2]*A[2][2]);                  // sigma33
}

TStringVector NoneParameterizationHeader(int n_var)
{
 int i, j, k=0;
 char buffer[32];
 TStringVector V=CreateStringVector(n_var*n_var+1);
 V[k++]=DuplicateString("None");
 for (j=0; j < n_var; j++)
  for (i=0; i < n_var; i++)
   {
    sprintf(buffer,"A(%d,%d)",i+1,j+1);
    V[k++]=DuplicateString(buffer);
   }
 return V;
}
TStringVector WZParameterizationHeader(void)
{
 return CreateStringVectorFromDelimited("WZ\tA(1,1)\tA(2,1)\tA(1,2)\tA(2,2)\tA(3,2)\tA(3,3)",'\t',0);
}
TStringVector WaldParameterization1stHeader(void)
{
 return CreateStringVectorFromDelimited("Wald (1st derivative)\tA(1,1)\tA(2,1)\tA(1,2)\tA(2,2)\tA(3,2)\tA(3,3)",'\t',0);
}
TStringVector WaldParameterization2ndHeader(void)
{
 return CreateStringVectorFromDelimited("Wald (2nd derivative)\tA(1,1)\tA(2,1)\tA(1,2)\tA(2,2)\tA(3,2)\tA(3,3)",'\t',0);
}
void AParameterization(TMatrix A, TVector v)
{
 v[0]=A[0][0];    // A(1,1)
 v[1]=A[1][0];    // A(2,1)
 v[2]=A[0][1];    // A(1,2)
 v[3]=A[1][1];    // A(2,2)
 v[4]=A[2][1];    // A(3,2)
 v[5]=A[2][2];    // A(3,3)
}

int ParameterizationDimension(file_setup_type *setup)
{
 switch (setup->parameterization)
   {
    case 0: return setup->n_var*setup->n_var;     // No normalization
    case 1: return 6;                             // WZ
    case 2: return 6;                             // Beta
    case 3: return 6;                             // Eta
    case 4: return 6;                             // Pi
    case 5: return 6;                             // Wald (1st derivative)
    case 6: return 6;                             // Wald (2nd derivative)
    default: printf("Unknown parameterization\n"); getchar(); exit(0);
    }
}

char* ParameterizationTypeString(int parameterization)
{
 switch (parameterization)
   {
    case 0: return "None";     // No normalization
    case 1: return "WZ";       // WZ
    case 2: return "Beta";     // Beta
    case 3: return "Eta";      // Eta
    case 4: return "Pi";       // Pi
    case 5: return "Wald (1st derivative)";     // Wald (1st derivative)
    case 6: return "Wald (2nd derivative)";     // Wald (2nd derivative)
    default: printf("Unknown parameterization\n"); getchar(); exit(0);
    }
}
              
TStringVector ParameterizationHeaders(file_setup_type *setup)
{
 switch (setup->parameterization)
   {
    case 0: return NoneParameterizationHeader(setup->n_var);     // No normalization
    case 1: return WZParameterizationHeader();                   // WZ
    case 2: return BetaParameterizationHeader();                 // Beta
    case 3: return EtaParameterizationHeader();                  // Eta
    case 4: return PiParameterizationHeader();                   // Pi
    case 5: return WaldParameterization1stHeader();              // Wald (1st derivative)
    case 6: return WaldParameterization2ndHeader();              // Wald (2nd derivative)
    default: printf("Unknown parameterization\n"); getchar(); exit(0);
    }
}

void Parameterization(TMatrix A, TVector v, int parameterization)
{
  switch (parameterization)
    {
    case 0: Vec(v,A); break;                    // No normalization
    case 1: AParameterization(A,v); break;      // WZ
    case 2: BetaParameterization(A,v); break;   // Beta
    case 3: EtaParameterization(A,v); break;    // Eta
    case 4: PiParameterization(A,v); break;     // Pi
    case 5:                                     // Wald (1st derivative)
    case 6: AParameterization(A,v); break;      // Wald (1st derivative)
    default: printf("Unknown parameterization\n"); getchar(); exit(0);
    }
}

/* #define NUMBER_PARAMETERIZATIONS 2 */
/* void AlternateParameterzations(FILE *f, T_SVAR *svar, TMatrix A_hat, TMatrix S_exact, TMatrix S_data) */
/* { */
/*  TMatrix A; */
/*  TVector v; */
/*  TMatrix *data; */
/*  int i, j, sample_size=20000; */

/*  data=CreateMatrixArray(NUMBER_PARAMETERIZATIONS); */
/*  for (i=0; i < NUMBER_PARAMETERIZATIONS; i++) */
/*   data[i]=CreateMatrix(6,sample_size); */

/*  v=CreateVector(6); */
/*  A=CreateMatrix(3,3); */

/*  SetNormalization(NORMALIZATION_NONE,(TMatrix)NULL,svar); */
/*  RandomInitialValue(A,100,svar); */

/*  TraditionalNormalization(A,v); for (i=5; i >= 0; i--) data[0][i][0]=v[i]; */
/*  PiNormalization(A,v); for (i=5; i >= 0; i--) data[1][i][0]=v[i]; */
/*  for (j=0; j < sample_size; j++) */
/*   { */
/*    Simulate_A(A,svar); */
/*    TraditionalNormalization(A,v); for (i=5; i >= 0; i--) data[0][i][j]=v[i]; */
/*    PiNormalization(A,v); for (i=5; i >= 0; i--) data[1][i][j]=v[i]; */
/*   } */

/*  printf("printing...\n"); */
/*  PrintHistogramVector(f,TraditionalNormalizationHeader(),data[0]); */
/*  PrintHistogramVector(f,PiNormalizationHeader(),data[1]); */
 
/*  FreeMatrix(A); */
/*  FreeVector(v); */
/*  FreeMatrixArray(data); */
/* } */


